@@ -59,11 +59,7 @@
  * ecvt, fcvt and gcvt.
  */
 
-/* From: Id: sprint.c,v 1.5 1995/09/10 18:35:09 chuck Exp */
-
-#ifdef HAVE_CONFIG_H
-# include <config.h>
-#endif
+#include <config.h>
 
 #ifndef HAVE_STDARG_H
 #error Need stdarg here!
