@@ -1,6 +1,6 @@
 /*
  * dial.c	Functions to dial, retry etc. Als contains the dialing
- *		directory code, _and_ the famous tu-di-di music.
+ *		directory code.
  *
  *		This file is part of the minicom communications package,
  *		Copyright 1991-1995 Miquel van Smoorenburg.
@@ -26,10 +26,7 @@
  *   jl 01.09.99  Move entry up/down in directory
  *   jl 10.02.2000 Stopbits field added
  */
-#ifdef HAVE_CONFIG_H
 #include <config.h>
-#endif
-
 #include <stdint.h>
 #include <limits.h>
 #include <arpa/inet.h>
@@ -38,14 +35,6 @@
 #include "minicom.h"
 #include "intl.h"
 
-#ifdef VC_MUSIC
-#  if defined(__GLIBC__)
-#    include <sys/ioctl.h>
-#    include <sys/kd.h>
-#    include <sys/time.h>
-#  endif
-#endif
-
 enum { CURRENT_VERSION = 6 };
 
 /* Dialing directory. */
@@ -145,16 +134,13 @@ static int nrents = 1;
 static int newtype;
 /* Access to ".dialdir" denied? */
 static int dendd = 0;
-static char *tagged;
-char *dial_user;
-char *dial_pass;
 
 /* Change the baud rate.  Treat all characters in the given array as if
  * they were key presses within the comm parameters dialog (C-A P) and
  * change the line speed accordingly.  Terminate when a space or other
  * unrecognised character is found.
  */
-const char* change_baud(const char *s)
+static const char* change_baud(const char *s)
 {
   while (s && *s
          && update_bbp_from_char(*s, P_BAUDRATE, P_BITS, P_PARITY, P_STOPB, 0))
@@ -163,8 +149,7 @@ const char* change_baud(const char *s)
   // reinitialise the port and update the status line
   if (portfd >= 0)
     port_init();
-  if (st)
-    mode_status();
+  show_status();
 
   return s;
 }
@@ -178,7 +163,7 @@ const char* change_baud(const char *s)
  * If how == 0, '~'  sleeps 1 second.
  * If how == 1, "^~" sleeps 1 second.
  */
-void mputs(const char *s, int how)
+static void mputs(const char *s, int how)
 {
   char c;
 
@@ -232,7 +217,8 @@ void mputs(const char *s, int how)
     if (how == 0 && c == '~')
       sleep(1);
     else
-      write(portfd, &c, 1);
+      if (write(portfd, &c, 1) != 1)
+        break;
     s++;
   }
 }
@@ -302,7 +288,7 @@ void hangup(void)
   /* If we don't have DCD support fake DCD dropped */
   bogus_dcd = 0;
   mc_wclose(w, 1);
-  time_status(false);
+  show_status();
 }
 
 /*
@@ -323,76 +309,29 @@ void sendbreak()
 WIN *dialwin;
 int dialtime;
 
-#ifdef VC_MUSIC
-
-/*
- * Play music until key is pressed.
- */
-void music(void)
-{
-  int x, i, k;
-  int consolefd = 0;
-  char *disp;
-
-  /* If we're in X, we have to explicitly use the console */
-  if (strncmp(getenv("TERM"), "xterm", 5) == 0 &&
-      (disp = getenv("DISPLAY")) != NULL &&
-      (strcmp(disp, ":0.0") == 0 ||
-       (strcmp(disp, ":0") == 0))) {
-    consolefd = open("/dev/console", O_WRONLY);
-    if (consolefd < 0) consolefd = 0;
-  }
-
-  /* Tell keyboard handler what we want. */
-  keyboard(KSIGIO, 0);
-
-  /* And loop forever :-) */
-  for(i = 0; i < 9; i++) {
-    k = 2000 - 200 * (i % 3);
-    ioctl(consolefd, KIOCSOUND, k);
-
-    /* Check keypress with timeout 160 ms */
-    x = check_io(-1, 0, 160, NULL, 0, NULL);
-    if (x & 2)
-      break;
-  }
-  ioctl(consolefd, KIOCSOUND, 0);
-  if (consolefd)
-    close(consolefd);
-
-  /* Wait for keypress and absorb it */
-  while ((x & 2) == 0) {
-    x = check_io(-1, 0, 10000, NULL, 0, NULL);
-    timer_update();
-  }
-  keyboard(KGETKEY, 0);
-}
-#endif
-
 /*
  * The dial has failed. Tell user.
  * Count down until retrytime and return.
  */
 static int dialfailed(char *s, int rtime)
 {
-  int f, x;
+  int f;
   int ret = 0;
 
   mc_wlocate(dialwin, 1, 5);
   mc_wprintf(dialwin, _("    No connection: %s.      \n"), s);
   if (rtime < 0) {
     mc_wprintf(dialwin, _("   Press any key to continue..    "));
-    if (check_io(-1, 0, 10000, NULL, 0, NULL) & 2)
+    if (check_io_input(10000))
       keyboard(KGETKEY, 0);
     return 0;
   }
   mc_wprintf(dialwin, _("     Retry in %2d seconds             "), rtime);
 
   for (f = rtime - 1; f >= 0; f--) {
-    x = check_io(-1, 0, 1000, NULL, 0, NULL);
-    if (x & 2) {
+    if (check_io_input(1000)) {
       /* Key pressed - absorb it. */
-      x = keyboard(KGETKEY, 0);
+      int x = keyboard(KGETKEY, 0);
       if (x != ' ')
         ret = -1;
       break;
@@ -527,7 +466,7 @@ MainLoop:
     buf[0] = 0;
     while (dialtime > 0) {
       if (*s == 0) {
-        x = check_io(portfd_connected, 0, 1000, buf, sizeof(buf), NULL);
+        x = check_io_frontend(buf, sizeof(buf), NULL);
         s = buf;
       }
       if (x & 2) {
@@ -564,7 +503,7 @@ MainLoop:
         /* Only look when we got a whole line. */
         if (modidx == 0 &&
             !strncmp(modbuf, P_MCONNECT, strlen(P_MCONNECT))) {
-          timer_update(); /* the login scipt may take long.. */
+          timer_update(); /* the login script may take long.. */
           retst = 0;
           /* Try to do auto-bauding */
           if (sscanf(modbuf + strlen(P_MCONNECT), "%ld", &nb) == 1)
@@ -580,11 +519,13 @@ MainLoop:
             do_log("%s %s, %s",modbuf, d->name, d->number);
 
           ptime = localtime(&now);
-          sprintf(d->lastdate,"%4.4d%2.2d%2.2d",
-                  (ptime->tm_year)+1900,(ptime->tm_mon)+1,
-                  ptime->tm_mday);
-          sprintf(d->lasttime,"%02d:%02d",
-                  ptime->tm_hour,ptime->tm_min);
+          snprintf(d->lastdate, sizeof(d->lastdate),
+	           "%4.4d%2.2d%2.2d",
+                   ptime->tm_year + 1900, ptime->tm_mon + 1,
+                   ptime->tm_mday);
+          snprintf(d->lasttime, sizeof(d->lasttime),
+	           "%02d:%02d",
+                   ptime->tm_hour, ptime->tm_min);
           d->count++;
 
           if (d->convfile[0]) {
@@ -596,22 +537,7 @@ MainLoop:
           if (d->script[0] == 0) {
             mc_wputs(dialwin,
                   _("Connected. Press any key to continue"));
-#ifdef VC_MUSIC
-            if (P_SOUND[0] == 'Y')
-              music();
-            else {
-              x = check_io(-1, 0, 0, NULL, 0, NULL);
-              if ((x & 2) == 2)
-                keyboard(KGETKEY, 0);
-            }
-#else
-            /* MARK updated 02/17/94 - If VC_MUSIC is not */
-            /* defined, then at least make some beeps! */
-            if (P_SOUND[0] == 'Y')
-              mc_wputs(dialwin,"\007\007\007");
-#endif
-            x = check_io(-1, 0, 0, NULL, 0, NULL);
-            if ((x & 2) == 2)
+            if (check_io_input(0))
               keyboard(KGETKEY, 0);
           }
           keyboard(KSTOP, 0);
@@ -740,26 +666,31 @@ static void convert_to_host_order(struct dialent *dst, const struct dialent *src
 }
 
 /* Read version 5 of the dialing directory. */
-void v5_read(FILE *fp, struct dialent *d)
+static int v5_read(FILE *fp, struct dialent *d)
 {
   struct dialent dent_n;
-  fread(&dent_n, sizeof(dent_n), 1, fp);
+  if (fread(&dent_n, sizeof(dent_n), 1, fp) != 1)
+    return 1;
   convert_to_host_order(d, &dent_n);
+  return 0;
 }
 
-void v6_read(FILE *fp, struct dialent *d)
+static int v6_read(FILE *fp, struct dialent *d)
 {
   struct dialent dent_n;
-  fread(&dent_n, sizeof(dent_n) - sizeof(void *), 1, fp);
+  if (fread(&dent_n, sizeof(dent_n) - sizeof(void *), 1, fp) != 1)
+    return 1;
   convert_to_host_order(d, &dent_n);
+  return 0;
 }
 
 /* Read version 4 of the dialing directory. */
-void v4_read(FILE *fp, struct dialent *d, struct dver *dv)
+static int v4_read(FILE *fp, struct dialent *d, struct dver *dv)
 {
   struct v4_dialent v4;
 
-  fread(&v4, dv->size, 1, fp);
+  if (fread(&v4, dv->size, 1, fp) != 1)
+    return 1;
 
   if (dv->size < sizeof(struct dialent)) {
     if (dv->size < offsetof(struct dialent, count) + sizeof(struct dialent *)) {
@@ -771,14 +702,18 @@ void v4_read(FILE *fp, struct dialent *d, struct dver *dv)
       d->convfile[0] = 0;
     strcpy(d->stopb, "1");
   }
+
+  return 0;
 }
 
 /* Read version 3 of the dialing directory. */
-void v3_read(FILE *fp, struct dialent *d)
+static int v3_read(FILE *fp, struct dialent *d)
 {
   struct v3_dialent v3;   /* jl 22.06.97 */
 
-  fread(&v3, sizeof(v3), 1, fp);
+  if (fread(&v3, sizeof(v3), 1, fp) != 1)
+    return 1;
+
   memcpy(d, &v3, offsetof(struct v3_dialent, next));
 
   d->lastdate[0] = 0;
@@ -786,14 +721,18 @@ void v3_read(FILE *fp, struct dialent *d)
   d->count = 0;
   d->convfile[0] = 0;
   strcpy(d->stopb, "1");
+
+  return 0;
 }
 
 /* Read version 2 of the dialing directory. */
-void v2_read(FILE *fp, struct dialent *d)
+static int v2_read(FILE *fp, struct dialent *d)
 {
   struct v3_dialent v3;   /* jl 22.06.97 */
 
-  fread((char *)&v3, sizeof(v3), 1, fp);
+  if (fread((char *)&v3, sizeof(v3), 1, fp) != 1)
+    return 1;
+
   memcpy(d, &v3, offsetof(struct v3_dialent, next));
   if (d->flags & FL_ANSI)
     d->flags |= FL_WRAP;
@@ -802,31 +741,45 @@ void v2_read(FILE *fp, struct dialent *d)
   d->count = 0;
   d->convfile[0] = 0;
   strcpy(d->stopb, "1");
+
+  return 0;
 }
 
 /* Read version 1 of the dialing directory. */
-void v1_read(FILE *fp, struct dialent *d)
+static int v1_read(FILE *fp, struct dialent *d)
 {
   struct v1_dialent v1;
 
-  fread((char *)&v1, sizeof(v1), (size_t)1, fp);
+  if (fread((char *)&v1, sizeof(v1), (size_t)1, fp) != 1)
+    return 1;
+
   memcpy(d->username, v1.username, sizeof(v1) - offsetof(struct v1_dialent, username));
-  strncpy(d->name, v1.name, sizeof(d->name));
+  _Static_assert(sizeof(d->name) >= sizeof(v1.name), "Size mismatch");
+  v1.name[sizeof(v1.name) - 1] = 0;
+  strcpy(d->name, v1.name);
+  d->name[sizeof(d->name) - 1] = '\0';
   strncpy(d->number, v1.number, sizeof(d->number));
+  d->number[sizeof(d->number) - 1] = '\0';
   strncpy(d->script, v1.script, sizeof(d->script));
+  d->script[sizeof(d->script) - 1] = '\0';
   d->lastdate[0]=0;
   d->lasttime[0]=0;
   d->count=0;
   d->convfile[0]=0;
   strcpy(d->stopb, "1");
+
+  return 0;
 }
 
 /* Read version 0 of the dialing directory. */
-void v0_read(FILE *fp, struct dialent *d)
+static int v0_read(FILE *fp, struct dialent *d)
 {
-  v1_read(fp, d);
+  if (v1_read(fp, d))
+    return 1;
   d->dialtype = 0;
   d->flags = 0;
+
+  return 0;
 }
 
 /*
@@ -848,10 +801,6 @@ int readdialdir(void)
     return 0;
   didread = 1;
   nrents = 1;
-  tagged = malloc(1);
-  if (!tagged)
-    return 0;
-  tagged[0] = 0;
 
   /* Make the manual dial entry. */
   d_man = mkstdent();
@@ -884,7 +833,12 @@ int readdialdir(void)
 
   /* Get version of the dialing directory */
   fseek(fp, 0L, SEEK_SET);
-  fread(&dial_ver, sizeof(dial_ver), 1, fp);
+  if (fread(&dial_ver, sizeof(dial_ver), 1, fp) != 1)
+    {
+      werror(_("Failed to read dialing directory\n"));
+      fclose(fp);
+      return -1;
+    }
   if (dial_ver.magic != DIALMAGIC) {
     /* First version without version info. */
     dial_ver.version = 0;
@@ -894,7 +848,7 @@ int readdialdir(void)
     size -= sizeof(dial_ver);
   }
 
-  /* See if the size of the file is allright. */
+  /* See if the size of the file is alright. */
   switch (dial_ver.version) {
     case 0:
     case 1:
@@ -918,12 +872,14 @@ int readdialdir(void)
           dial_ver.size > sizeof(struct v4_dialent)) {
         werror(_("Phonelist garbled (unknown version?)"));
         dialents = mkstdent();
+        fclose(fp);
         return -1;
       }
       break;
     case 5:
       if (dial_ver.size != sizeof(struct dialent)) {
 	werror(_("Phonelist corrupted"));
+        fclose(fp);
 	return -1;
       }
       break;
@@ -932,6 +888,7 @@ int readdialdir(void)
       // have different size on 32 and 64bit systems
       if (dial_ver.size != sizeof(struct dialent) - sizeof(void *)) {
         werror(_("Phonelist corrupted"));
+        fclose(fp);
         return -1;
       }
       break;
@@ -939,6 +896,7 @@ int readdialdir(void)
       werror(_("Unknown dialing directory version"));
       dendd = 1;
       dialents = mkstdent();
+      fclose(fp);
       return -1;
   }
 
@@ -969,29 +927,34 @@ int readdialdir(void)
       fclose(fp);
       return -1;
     }
+    int r = 0;
     switch (dial_ver.version) {
       case 0:
-        v0_read(fp, d);
+        r = v0_read(fp, d);
         break;
       case 1:
-        v1_read(fp, d);
+        r = v1_read(fp, d);
         break;
       case 2:
-        v2_read(fp, d);
+        r = v2_read(fp, d);
         break;
       case 3:
-        v3_read(fp, d);
+        r = v3_read(fp, d);
         break;
       case 4:
-        v4_read(fp, d, &dial_ver);
+        r = v4_read(fp, d, &dial_ver);
         break;
       case 5:
-	v5_read(fp, d);
+	r = v5_read(fp, d);
 	break;
       case 6:
-	v6_read(fp, d);
+	r = v6_read(fp, d);
 	break;
     }
+
+    if (r)
+      werror(_("Failed to read phonelist\n"));
+
     /* MINIX terminal type is obsolete */
     if (d->term == 2)
       d->term = 1;
@@ -1115,8 +1078,8 @@ static const char *te[] = { "VT102", "MINIX", "ANSI " };
 static void dedit_toggle_entry(WIN *w, int x, int y, int toggle,
                                char *toggle_true, char *toggle_false)
 {
-  int lt = mbslen(toggle_true);
-  int lf = mbslen(toggle_false);
+  int lt = mbswidth(toggle_true);
+  int lf = mbswidth(toggle_false);
   int l = ((lt > lf) ? lt : lf) + 1;
   char *buf, *s = toggle ? toggle_true : toggle_false;
   int i;
@@ -1125,7 +1088,7 @@ static void dedit_toggle_entry(WIN *w, int x, int y, int toggle,
     return;
 
   strncpy(buf, s, l);
-  for (i = mbslen(s); i < l - 1; i++)
+  for (i = mbswidth(s); i < l - 1; i++)
     buf[i] = ' ';
   buf[l - 1] = 0;
 
@@ -1177,7 +1140,7 @@ static void dedit(struct dialent *d)
   mc_wredraw(w, 1);
 
   while (1) {
-    mc_wlocate(w, mbslen (question) + 5, 15);
+    mc_wlocate(w, mbswidth(question) + 5, 15);
     c = wxgetch();
     if (c >= 'a')
       c -= 32;
@@ -1188,35 +1151,35 @@ static void dedit(struct dialent *d)
         mc_wclose(w, 1);
         return;
       case 'A':
-        mc_wlocate(w, mbslen (name) + 1, 0);
+        mc_wlocate(w, mbswidth(name) + 1, 0);
         mc_wgets(w, d->name, 31, 32);
         break;
       case 'B':
-        mc_wlocate(w, mbslen (number) + 1, 1);
+        mc_wlocate(w, mbswidth(number) + 1, 1);
         mc_wgets(w, d->number, 31, 32);
         break;
       case 'C':
         d->dialtype = (d->dialtype + 1) % 3;
-        mc_wlocate(w, mbslen (dial_string) + 1, 2);
+        mc_wlocate(w, mbswidth(dial_string) + 1, 2);
         mc_wprintf(w, "%d", d->dialtype + 1);
         mc_wflush();
         break;
       case 'D':
         d->flags ^= FL_ECHO;
-        mc_wlocate(w, mbslen (local_echo_str) + 1, 3);
+        mc_wlocate(w, mbswidth(local_echo_str) + 1, 3);
         mc_wprintf(w, "%s", _(yesno(d->flags & FL_ECHO)));
         mc_wflush();
         break;
       case 'E':
-        mc_wlocate(w, mbslen (script) + 1, 4);
+        mc_wlocate(w, mbswidth(script) + 1, 4);
         mc_wgets(w, d->script, 31, 32);
         break;
       case 'F':
-        mc_wlocate(w, mbslen (username) + 1, 5);
+        mc_wlocate(w, mbswidth(username) + 1, 5);
         mc_wgets(w, d->username, 31, 32);
         break;
       case 'G':
-        mc_wlocate(w, mbslen (password) + 1, 6);
+        mc_wlocate(w, mbswidth(password) + 1, 6);
         mc_wgets(w, d->password, 31, 32);
         break;
       case 'H':
@@ -1224,7 +1187,7 @@ static void dedit(struct dialent *d)
         /* MINIX == 2 is obsolete. */
         if (d->term == 2)
           d->term = 3;
-        mc_wlocate(w, mbslen (terminal_emulation) + 1, 7);
+        mc_wlocate(w, mbswidth(terminal_emulation) + 1, 7);
         mc_wputs(w, te[d->term - 1]);
 
         /* Also set backspace key. */
@@ -1235,29 +1198,29 @@ static void dedit(struct dialent *d)
           d->flags |= FL_DEL;
           d->flags &= ~FL_WRAP;
         }
-	dedit_toggle_entry(w, mbslen(backspace_key_sends) + 1, 8,
+	dedit_toggle_entry(w, mbswidth(backspace_key_sends) + 1, 8,
 	                   d->flags & FL_DEL, _("Delete"), _("Backspace"));
-	dedit_toggle_entry(w, mbslen(linewrap) + 1, 9,
+	dedit_toggle_entry(w, mbswidth(linewrap) + 1, 9,
 	                   d->flags & FL_WRAP, _("On"), _("Off"));
         break;
       case 'I':
         d->flags ^= FL_DEL;
-	dedit_toggle_entry(w, mbslen(backspace_key_sends) + 1, 8,
+	dedit_toggle_entry(w, mbswidth(backspace_key_sends) + 1, 8,
 	                   d->flags & FL_DEL, _("Delete"), _("Backspace"));
         break;
       case 'J':
         d->flags ^= FL_WRAP;
-	dedit_toggle_entry(w, mbslen(linewrap) + 1, 9,
+	dedit_toggle_entry(w, mbswidth(linewrap) + 1, 9,
 	                   d->flags & FL_WRAP, _("On"), _("Off"));
         break;
       case 'K':
         get_bbp(d->baud, d->bits, d->parity, d->stopb, 1);
-        mc_wlocate(w, mbslen (line_settings) + 1, 10);
+        mc_wlocate(w, mbswidth(line_settings) + 1, 10);
         mc_wprintf(w, "%s %s%s%s  ",
                 d->baud, d->bits, d->parity, d->stopb);
         break;
       case 'L':	/* jl 21.09.97 */
-        mc_wlocate(w, mbslen (conversion_table) + 1, 11);
+        mc_wlocate(w, mbswidth(conversion_table) + 1, 11);
         mc_wgets(w, d->convfile, 15, 16);
         break;
       default:
@@ -1434,10 +1397,10 @@ static void dial_entry(struct dialent *d)
     strcpy(P_BITS, d->bits);
     strcpy(P_STOPB, d->stopb);
     port_init();
-    mode_status();
+    show_status();
   }
   newtype = d->term;
-  vt_set(-1, d->flags & FL_WRAP, -1, -1, d->flags & FL_ECHO, -1, -1, -1);
+  vt_set(-1, d->flags & FL_WRAP, -1, -1, d->flags & FL_ECHO, -1, -1, -1, -1);
   local_echo = d->flags & FL_ECHO;
   if (newtype != terminal)
     init_emul(newtype, 1);
@@ -1457,9 +1420,9 @@ static void dial_entry(struct dialent *d)
   if (P_MAUTOBAUD[0] == 'Y' && nb) {
     sprintf(P_BAUDRATE, "%ld", nb);
     port_init();
-    mode_status();
+    show_status();
   } else if (P_SHOWSPD[0] == 'l')
-    mode_status();
+    show_status();
 
   /* Make sure the last date is updated   / jl 22.06.97 */
   writedialdir();
@@ -1540,7 +1503,7 @@ void dialdir(void)
   int first = 1;
   int x1, x2;
   char *s, dname[128];
-  static char manual[128];
+  static char manual[32];
   int changed = 0;
   static const char *tag_exit  = N_("( Escape to exit, Space to tag )"),
                     *move_exit = N_(" Move entry up/down, Escape to exit");
@@ -1554,7 +1517,7 @@ void dialdir(void)
   if (strlen(_(tag_exit)) > tagmvlen)
     tagmvlen = strlen(_(tag_exit));
 
-  /* Allright, draw the dialing directory! */
+  /* Alright, draw the dialing directory! */
 
   dirflush = 0;
   x1 = (COLS / 2) - 37;
@@ -1725,7 +1688,7 @@ again:
           d = d1;
           break;
         }
-      /* ..and if none of them was tagged, check from the begining. */
+      /* ..and if none of them was tagged, check from the beginning. */
       if (!d1)
         for (d1 = dialents; d1 && d1!=d; d1 = d1->next)
           if (d1->flags & FL_TAG) {
@@ -1739,7 +1702,7 @@ again:
   }
   /* Find an entry */
   if (subm == 1) {
-    s = input(_("Find an entry"), dname);
+    s = input(_("Find an entry"), dname, sizeof(dname));
     if (s == NULL || s[0] == 0)
       goto again;
     x1 = 0;
@@ -1844,7 +1807,7 @@ again:
 
   /* Dial a number manually. */
   if (subm == 6) {
-    s = input(_("Enter number"), manual);
+    s = input(_("Enter number"), manual, sizeof(manual));
     if (s && *s) {
       if (changed)
         writedialdir();
@@ -1854,9 +1817,20 @@ again:
       strncpy(d_man->number, manual, sizeof(d_man->number));
       dial(d_man, (struct dialent**)NULL);
       if (P_SHOWSPD[0] == 'l')
-        mode_status();
+        show_status();
       return;
     }
   }
   goto again;
 }
+
+void free_dialents(void)
+{
+  struct dialent *d = dialents;
+
+  while (d) {
+    struct dialent *tmp = d;
+    d = d->next;
+    free(tmp);
+  }
+}
