@@ -5,8 +5,6 @@
  *	Datatypes and constants for getsdir() - a function to get and
  *	return a sorted directory listing.
  *
- *	$Id: getsdir.h,v 1.4 2007-10-10 20:18:20 al-guest Exp $
- *
  *	Copyright (c) 1998 by James S. Seymour (jseymour@jimsun.LinxNet.com)
  *
  *	This code is free software; you can redistribute it and/or
@@ -23,9 +21,10 @@
  */
 
 #include <dirent.h>
+#include <limits.h>
 
 typedef struct dirEntry {		/* structure of data item */
-  char fname[MAXNAMLEN + 1];		/* filename + terminating null */
+  char fname[NAME_MAX + 1];		/* filename + terminating null */
   time_t time;				/* last modification date */
   mode_t mode;				/* file mode (dir? etc.) */
   ushort cflags;			/* caller field for convenience */
