@@ -18,62 +18,73 @@
  *  with this program; if not, write to the Free Software Foundation, Inc.,
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
-#ifdef HAVE_CONFIG_H
 #include <config.h>
-#endif
-
 #include <strings.h>
 
 #include "port.h"
 #include "minicom.h"
 #include "intl.h"
 
-/* Check if there is IO pending. */
-int check_io(int fd1, int fd2, int tmout, char *buf,
-             int bufsize, int *bytes_read)
+int read_buf(int fd, char *buf, int bufsize)
 {
-  int n = 0, i;
-  struct timeval tv;
-  fd_set fds;
+  int i = read(fd, buf, bufsize - 1);
 
-  tv.tv_sec = tmout / 1000;
-  tv.tv_usec = (tmout % 1000) * 1000L;
-
-  i = fd1;
-  if (fd2 > fd1)
-    i = fd2;
-
-  FD_ZERO(&fds);
-  if (fd1 >= 0)
-    FD_SET(fd1, &fds);
-  else
-    fd1 = 0;
-  if (fd2 >= 0)
-    FD_SET(fd2, &fds);
-  else
-    fd2 = 0;
-
-  if (fd2 == 0 && io_pending)
-    n = 2;
-  else if (select(i + 1, &fds, NULL, NULL, &tv) > 0)
-    n = 1 * (FD_ISSET(fd1, &fds) > 0) + 2 * (FD_ISSET(fd2, &fds) > 0);
-
-  /* If there is data put it in the buffer. */
-  if (buf) {
+  if (i < 1 && portfd_is_socket && portfd == fd) {
+    term_socket_close();
     i = 0;
-    if ((n & 1) == 1) {
-      i = read(fd1, buf, bufsize - 1);
-#ifdef USE_SOCKET
-      if (!i && portfd_is_socket && portfd == fd1)
-        term_socket_close();
-#endif /* USE_SOCKET */
-    }
-    buf[i > 0 ? i : 0] = 0;
-    if (bytes_read)
-      *bytes_read = i;
   }
 
-  return n;
+  buf[i > 0 ? i : 0] = 0;
+
+  return i;
+}
+
+/* Check if there is IO pending. */
+static int check_io(int fd1, int fd2, int tmout, char *buf,
+                    int bufsize, int *bytes_read)
+{
+  struct timeval tv;
+  fd_set fds;
+  FD_ZERO(&fds);
+
+  tv.tv_sec  = tmout / 1000;
+  tv.tv_usec = (tmout % 1000) * 1000L;
+
+  int max_fd = fd1;
+  if (fd2 > fd1)
+    max_fd = fd2;
+
+  if (fd1 >= 0)
+    FD_SET(fd1, &fds);
+
+  if (fd2 >= 0)
+    FD_SET(fd2, &fds);
+
+  int ret = 0;
+  if (fd2 == STDIN_FILENO && io_pending)
+    ret = 2;
+  else if (select(max_fd + 1, &fds, NULL, NULL, &tv) > 0)
+    {
+      ret =   1 * (fd1 >= 0 && FD_ISSET(fd1, &fds) != 0)
+            + 2 * (fd2 >= 0 && FD_ISSET(fd2, &fds) != 0);
+
+      /* If there is data put it in the buffer. */
+      if (buf && bytes_read && (ret & 1))
+        *bytes_read = read_buf(fd1, buf, bufsize);
+    }
+
+  return ret;
+}
+
+int check_io_frontend(char *buf, int buf_size, int *bytes_read)
+{
+  return check_io(portfd_connected(), STDIN_FILENO, 1000,
+                  buf, buf_size, bytes_read);
+}
+
+bool check_io_input(int timeout_ms)
+{
+  return check_io(-1, STDIN_FILENO, timeout_ms, NULL, 0, NULL) & 2;
 }
 
 int keyboard(int cmd, int arg)
@@ -90,13 +101,13 @@ int keyboard(int cmd, int arg)
       escape = arg;
       break;
     case KSETBS:
-      vt_set(-1, -1, -1, arg, -1, -1, -1, -1);
+      vt_set(-1, -1, -1, arg, -1, -1, -1, -1, -1);
       break;
     case KCURST:
-      vt_set(-1, -1, -1, -1, -1, NORMAL, -1, -1);
+      vt_set(-1, -1, -1, -1, -1, NORMAL, -1, -1, -1);
       break;
     case KCURAPP:
-      vt_set(-1, -1, -1, -1, -1, APPL, -1, -1);
+      vt_set(-1, -1, -1, -1, -1, APPL, -1, -1, -1);
       break;
     default:
       /* The rest is only meaningful if a keyserv runs. */
