@@ -368,8 +368,6 @@ void device_close()
  */
 int open_term(int doinit, int show_win_on_error, int no_msgs)
 {
-  int s_errno;
-
   portfd_is_socket = portfd_is_connected = 0;
   size_t ulen = strlen(SOCKET_PREFIX_UNIX);
   assert(ulen == strlen(SOCKET_PREFIX_UNIX_LEGACY));
@@ -390,36 +388,41 @@ int open_term(int doinit, int show_win_on_error, int no_msgs)
 
 nolock:
   /* Run a special program to disable callin if needed. */
-    if (doinit > 0 && P_CALLOUT[0]) {
-      if (fastsystem(P_CALLOUT, NULL, NULL, NULL) < 0) {
-        if (stdwin)
-          mc_wclose(stdwin, 1);
-        fprintf(stderr, _("Could not setup for dial out.\n"));
-	lockfile_remove();
-        return -1;
-      }
+  if (doinit > 0 && P_CALLOUT[0]) {
+    if (fastsystem(P_CALLOUT, NULL, NULL, NULL) < 0) {
+      if (stdwin)
+        mc_wclose(stdwin, 1);
+      fprintf(stderr, _("Could not setup for dial out.\n"));
+      lockfile_remove();
+      return -1;
     }
+  }
 
   /* Now open the tty device. */
   if (setjmp(albuf) == 0) {
     portfd = -1;
     signal(SIGALRM, get_alrm);
     alarm(20);
+
     if (portfd_is_socket)
       term_socket_connect();
     else
+      device_open();
+
+    if (portfd < 0)
       {
-        if (device_open())
-	  return -1;
+	if (!no_msgs)
+          fprintf(stderr, _("minicom: cannot open %s: %s\n"),
+                          dial_tty, strerror(errno)); // errno retained from callers
+        return -1;
       }
 
-    if (portfd >= 0) {
-      if (doinit > 0)
-        m_savestate(portfd);
-      port_init();
-    }
+    if (doinit > 0)
+      m_savestate(portfd);
+    port_init();
   }
-  s_errno = errno;
+
+  int s_errno = errno;
   alarm(0);
   signal(SIGALRM, SIG_IGN);
   if (portfd < 0 && portfd_is_socket == Socket_type_no_socket) {
