@@ -1,8 +1,6 @@
 /*
  * minicom.h	Constants, defaults, globals etc.
  *
- *		$Id: minicom.h,v 1.26 2009-06-06 21:19:36 al-guest Exp $
- *
  *		This file is part of the minicom communications package,
  *		Copyright 1991-1995 Miquel van Smoorenburg.
  *
@@ -25,7 +23,7 @@
  *    for new "get_port()" function in util.c.
  */
 
-/* First include all other application-dependant include files. */
+/* First include all other application-dependent include files. */
 #include "config.h"
 #include "configsym.h"
 #include "window.h"
@@ -34,17 +32,15 @@
 #include "libport.h"
 
 #include <time.h>
-
-#ifdef USE_SOCKET
+#include <stdbool.h>
+#include <sys/types.h>
 #include <sys/socket.h>
 #include <sys/un.h>
-#endif
+#include <arpa/inet.h>
 
-/*
- * kubota@debian.or.jp 08/08/98
- * COLS must be equal to or less than MAXCOLS.
- */
-#define MAXCOLS 256
+#if HAVE_LOCKDEV
+#include <ttylock.h>
+#endif
 
 #define XA_OK_EXIST	1
 #define XA_OK_NOTEXIST	2
@@ -58,11 +54,12 @@ EXTERN int LINES;
 EXTERN int COLS;
 #endif
 
-EXTERN int dosetup;
+EXTERN int dosetup;     /* In setup mode (-s) */
 
 EXTERN char stdattr;	/* Standard attribute */
 
 EXTERN WIN *us;		/* User screen */
+EXTERN WIN *us_alternate; /* Alternate user screen */
 EXTERN WIN *st;		/* Status Line */
 
 EXTERN short terminal;	/* terminal type */
@@ -76,13 +73,24 @@ EXTERN int cursormode;	/* Mode of cursor (arrow) keys */
 
 EXTERN int docap;	/* Capture data to capture file */
 EXTERN FILE *capfp;	/* File to capture to */
+EXTERN int capbuf;	/* Buffering mode of capture file */
 EXTERN int addlf;	/* Add LF after CR */
+EXTERN int addcr;	/* Insert CR before LF */
 EXTERN int wrapln;	/* Linewrap default */
+EXTERN int display_hex; /* Display in hex */
 EXTERN int tempst;	/* Status line is temporary */
 EXTERN int escape;	/* Escape code. */
-EXTERN int disable_online_time; /* disable online time display */
 
-EXTERN char lockfile[128]; /* UUCP lock file of terminal */
+enum Lockfile_mode
+{
+  Lockfile_mode_unset   = 0,
+  Lockfile_mode_uucp    = 1,
+  Lockfile_mode_flock   = 2,
+  Lockfile_mode_ttylock = 3,
+  Lockfile_mode_max     = 4,
+};
+EXTERN enum Lockfile_mode lockfile_mode;
+EXTERN char lockfile[270]; /* UUCP lock file of terminal */
 EXTERN char homedir[256];  /* Home directory of user */
 EXTERN char logfname[PARS_VAL_LEN]; /* Name of the logfile */
 EXTERN char username[16];  /* Who is using minicom? */
@@ -93,29 +101,29 @@ EXTERN int alt_override;/* -m option */
 EXTERN char parfile[256]; /* Global parameter file */
 EXTERN char pparfile[256]; /* Personal parameter file */
 
-EXTERN char scr_name[33];   /* Name of last script */
+EXTERN char scr_name[256];  /* Name of last script */
 EXTERN char scr_user[33];   /* Login name to use with script */
 EXTERN char scr_passwd[33]; /* Password to use with script */
 
 EXTERN char termtype[32];  /* Value of getenv("TERM"); */
 EXTERN char *dial_tty;     /* tty to use. */
 
-EXTERN char *dial_name;	    /* System we're conneced to */
+EXTERN char *dial_name;	    /* System we're connected to */
 EXTERN char *dial_number;   /* Number we've dialed. */
 EXTERN char *dial_user;     /* Our username there */
 EXTERN char *dial_pass;     /* Our password */
 
-#ifdef USE_SOCKET
-EXTERN int portfd_is_socket;	/* File descriptor is a unix socket */
-EXTERN int portfd_is_connected;	/* 1 if the socket is connected */
-EXTERN struct sockaddr_un portfd_sock_addr;	/* the unix socket address */
-#define portfd_connected ((portfd_is_socket && !portfd_is_connected) \
-                           ? -1 : portfd)
-#else
-#define portfd_connected portfd
-#define portfd_is_socket 0
-#define portfd_is_connected 0
-#endif /* USE_SOCKET */
+enum Socket_type {
+  Socket_type_no_socket = 0,
+  Socket_type_unix = 1,
+  Socket_type_tcp = 2,
+};
+extern enum Socket_type portfd_is_socket;	/* File descriptor is a unix socket */
+extern int portfd_is_connected;	/* 1 if the socket is connected */
+static inline int portfd_connected(void)
+{
+  return (portfd_is_socket && !portfd_is_connected) ? -1 : portfd;
+}
 
 /*
  * fmg 8/22/97
@@ -134,7 +142,7 @@ EXTERN int sbcolor;     /* Status Bar Background Color */
 EXTERN int st_attr;	/* Status Bar attributes. */
 
 /* jl 04.09.97 conversion tables */
-EXTERN unsigned char vt_outmap[256], vt_inmap[256];
+extern unsigned char vt_outmap[256], vt_inmap[256];
 
 /* MARK updated 02/17/95 - history buffer */
 EXTERN int num_hist_lines;  /* History buffer size */
@@ -144,7 +152,7 @@ EXTERN int num_hist_lines;  /* History buffer size */
 int Jcolor(char *);
 
 EXTERN int size_changed;     /* Window size has changed */
-extern const char *Version;  /* Minicom verson */
+extern const char *Version;  /* Minicom version */
 
 EXTERN int local_echo;      /* Local echo on/off. */
 
@@ -167,20 +175,15 @@ void vdodflsave(void);	/* fmg - need it */
 int  domacsave(void);	/* fmg - need it */
 int  loadconv(char *);	/* jl */
 int  saveconv(char *);	/* jl */
-int  speed_valid(unsigned int);
 
 /* Prototypes from file: common.c */
 char *pfix_home( char *s);
 void do_log(const char *line, ...);
 size_t one_mbtowc (wchar_t *pwc, const char *s, size_t n);
 size_t one_wctomb (char *s, wchar_t wchar);
-size_t mbslen (const char *s);
+size_t mbswidth(const char *s);
 
 /* Prototypes from file: dial.c */
-#if VC_MUSIC
-void music(void);
-#endif
-void mputs(const char *s , int how);
 void modeminit(void);
 void modemreset(void);
 void hangup(void);
@@ -189,6 +192,7 @@ long dial(struct dialent *d , struct dialent **d2);
 int  readdialdir(void);
 void dialone(char *entry);
 void dialdir(void);
+void free_dialents(void);
 
 /* Prototypes from file: file.c */
 char *filedir(int how_many, int downloading);
@@ -203,7 +207,9 @@ char *get_port(char *);
 int help(void);
 
 /* Prototypes from file: ipc.c */
-int check_io(int fd1, int fd2, int tmout, char *buf, int buf_size, int *bytes_red);
+int check_io_frontend(char *buf, int buf_size, int *bytes_red);
+bool check_io_input(int timeout_ms);
+int read_buf(int fd, char *buf, int bufsize);
 int keyboard(int cmd, int arg);
 
 /* Prototypes from file: keyserv.c */
@@ -219,18 +225,21 @@ void term_socket_close(void);
 int  open_term(int doinit, int show_win_on_error, int no_msgs);
 void init_emul(int type, int do_init);
 void timer_update(void);
-void mode_status(void);
-void time_status(void);
-void curs_status(void);
 void show_status(void);
+void set_status_line_format(const char *s);
 void scriptname(const char *s);
 int  do_terminal(void);
 void status_set_display(const char *text, int duration_s);
 
 /* Prototypes from file: minicom.c */
 void port_init(void);
+int convert_baudstring_to_uint(const char *s, unsigned *baudrate);
+void set_P_BAUDRATE(unsigned baudrate);
 void toggle_addlf(void);
 void toggle_local_echo(void);
+void set_local_echo(int val);
+void set_addlf(int val);
+void set_addcr(int val);
 
 void drawhist_look(WIN *w, int y, int r, wchar_t *look, int case_matters);
 void searchhist(WIN *w_hist, wchar_t *str);
@@ -262,8 +271,11 @@ void m_hupcl(int fd, int on);
 void m_flush(int fd);
 void m_flush_script( int fd);
 unsigned m_getmaxspd(void);
-void m_setparms(int fd, char *baudr, char *par, char *bits,
-                char *stopb, int hwf, int swf);
+bool m_setparms(int fd, int baudrate, char *par, char *bits, char *stopb,
+                int hwf, int swf, int rs485en);
+void m_set485parms(int fd, int en, int rts_on_snd, int rts_aft_snd,
+                   int rx_dur_tx, int term_bus, char *del_rts_bef_snd,
+                   char *del_rts_aft_snd);
 int  m_wait(int *st);
 
 /* Prototypes from file: sysdep2.c */
@@ -282,7 +294,7 @@ int  paste_file(void);
 WIN *mc_tell(const char *, ...);
 void werror(const char *, ...);
 int ask(const char *what, const char *s[]);
-char *input(char *s, char *buf);
+char *input(char *s, char *buf, size_t bufsize);
 
 /* Prototypes from file: wildmat.c */
 int wildmat(const char *, const char *);
@@ -293,10 +305,12 @@ extern int io_pending, pendingkeys;
 /* Prototypes from file: config.c */
 void domacros(void);
 
+void device_close(void);
 
-void lockfile_create(void);
+int lockfile_create(void);
 void lockfile_remove(void);
 
+#define ARRAY_SIZE(a) (sizeof(a) / sizeof(a[0]))
 
 
 /* We want the ANSI offsetof macro to do some dirty stuff. */
@@ -312,3 +326,21 @@ void lockfile_remove(void);
 #define FL_TAG		0x80	/* This entry is tagged. */
 #define FL_SAVE		0x0f	/* Which portions of flags to save. */
 
+enum {
+  TIMESTAMP_LINE_OFF,
+  TIMESTAMP_LINE_SIMPLE,
+  TIMESTAMP_LINE_EXTENDED,
+  TIMESTAMP_LINE_PER_SECOND,
+  TIMESTAMP_LINE_DELTA,
+  TIMESTAMP_LINE_NR_OF_OPTIONS, // must be last
+};
+
+int toggle_line_timestamp(void);
+const char *line_timestamp_description(void);
+const char *timestamp_option_idstring(const int o);
+
+
+/* Enhanced snprintf function -- as from Linux */
+int vscnprintf(char *buf, size_t size, const char *fmt, va_list args);
+int scnprintf(char *buf, size_t size, const char *fmt, ...)
+    __attribute__((format(printf, 3, 4)));
