@@ -1,7 +1,7 @@
 /*
- * sysdep1.c	system dependant routines.
+ * sysdep1.c	system dependent routines.
  *
- *		m_dtrtoggle	- dropt dtr and raise it again
+ *		m_dtrtoggle	- drop dtr and raise it again
  *		m_break		- send BREAK signal
  *		m_getdcd	- get modem dcd status
  *		m_setdcd	- set modem dcd status
@@ -29,13 +29,72 @@
  *
  *   jl  23.06.97  adjustable DTR downtime
  */
-#ifdef HAVE_CONFIG_H
 #include <config.h>
-#endif
 
+#include "port.h"
 #include "sysdep.h"
 #include "minicom.h"
 
+#ifdef __APPLE__
+#include <IOKit/serial/ioss.h>
+#endif
+
+/*
+ * This is for supporting higher baud rates on MacOS.
+ * Reference:
+ * https://github.com/npat-efault/picocom/blob/master/custbaud_bsd.h */
+#ifndef B460800
+#define B460800   460800
+#endif
+#ifndef B500000
+#define B500000   500000
+#endif
+#ifndef B576000
+#define B576000   576000
+#endif
+#ifndef B921600
+#define B921600   921600
+#endif
+#ifndef B1000000
+#define B1000000 1000000
+#endif
+#ifndef B1152000
+#define B1152000 1152000
+#endif
+#ifndef B1500000
+#define B1500000 1500000
+#endif
+#ifndef B2000000
+#define B2000000 2000000
+#endif
+#ifndef B2500000
+#define B2500000 2500000
+#endif
+#ifndef B3000000
+#define B3000000 3000000
+#endif
+#ifndef B3500000
+#define B3500000 3500000
+#endif
+#ifndef B4000000
+#define B4000000 4000000
+#endif
+
+/* On MacOS, setting unusual baudrates (see list above) does not work
+ * through tcsetattr but requires an additional ioctl. Interestingly,
+ * tcgetattr then returns the right baud rate, so we do not need to do
+ * anything there. */
+static void set_speed_apple(int fd, struct termios *p)
+{
+#ifdef __APPLE__
+  speed_t v = cfgetospeed(p);
+  ioctl(fd, IOSSIOSPEED, &v);
+#else
+  (void)fd;
+  (void)p;
+#endif
+}
+
 /* Set hardware flow control. */
 void m_sethwf(int fd, int on)
 {
@@ -43,10 +102,8 @@ void m_sethwf(int fd, int on)
   struct termios tty;
 #endif
 
-#ifdef USE_SOCKET
   if (portfd_is_socket)
 	return;
-#endif
 
 #ifdef POSIX_TERMIOS
   tcgetattr(fd, &tty);
@@ -55,16 +112,16 @@ void m_sethwf(int fd, int on)
   else
     tty.c_cflag &= ~CRTSCTS;
   tcsetattr(fd, TCSANOW, &tty);
+  set_speed_apple(fd, &tty);
 #endif
 }
 
 /* Set RTS line. Sometimes dropped. Linux specific? */
 static void m_setrts(int fd)
 {
-#ifdef USE_SOCKET
   if (portfd_is_socket)
     return;
-#endif
+
 #if defined(TIOCM_RTS) && defined(TIOCMODG)
   {
     int mcs=0;
@@ -81,19 +138,20 @@ static void m_setrts(int fd)
  */
 void m_dtrtoggle(int fd, int sec)
 {
-#ifdef USE_SOCKET
   if (portfd_is_socket)
     return;
-#endif
+
   {
 #ifdef TIOCSDTR
     /* Use the ioctls meant for this type of thing. */
     ioctl(fd, TIOCCDTR, 0);
-    if (sec>0) {
-      sleep(sec);
-      ioctl(fd, TIOCSDTR, 0);
-    }
-
+    sleep(sec);
+    ioctl(fd, TIOCSDTR, 0);
+#elif defined (TIOCMBIS) && defined (TIOCMBIC) && defined (TIOCM_DTR)
+    int modembits = TIOCM_DTR;
+    ioctl(fd, TIOCMBIC, &modembits);
+    sleep (sec);
+    ioctl(fd, TIOCMBIS, &modembits);
 #else /* TIOCSDTR */
 #  if defined (POSIX_TERMIOS)
 
@@ -105,10 +163,9 @@ void m_dtrtoggle(int fd, int sec)
     cfsetospeed(&tty, B0);
     cfsetispeed(&tty, B0);
     tcsetattr(fd, TCSANOW, &tty);
-    if (sec > 0) {
-      sleep(sec);
-      tcsetattr(fd, TCSANOW, &old);
-    }
+    sleep(sec);
+    tcsetattr(fd, TCSANOW, &old);
+    set_speed_apple(fd, &old);
 
 #  else /* POSIX */
 #    ifdef _V7
@@ -121,10 +178,8 @@ void m_dtrtoggle(int fd, int sec)
 
     ng.sg_ispeed = ng.sg_ospeed = 0;
     ioctl(fd, TIOCSETP, &ng);
-    if (sec > 0) {
-      sleep(sec);
-      ioctl(fd, TIOCSETP, &sg);
-    }
+    sleep(sec);
+    ioctl(fd, TIOCSETP, &sg);
 
 #    endif /* _V7 */
 #  endif /* POSIX */
@@ -137,10 +192,9 @@ void m_dtrtoggle(int fd, int sec)
  */
 void m_break(int fd)
 {
-#ifdef USE_SOCKET
   if (portfd_is_socket)
     return;
-#endif
+
 #ifdef POSIX_TERMIOS
   tcsendbreak(fd, 0);
 #else
@@ -171,7 +225,6 @@ void m_break(int fd)
  */
 int m_getdcd(int fd)
 {
-#ifdef USE_SOCKET
   if (portfd_is_socket) {
     if (portfd_is_connected)
       return 1;
@@ -179,7 +232,7 @@ int m_getdcd(int fd)
     term_socket_connect();
     return portfd_is_connected;
   }
-#endif
+
 #ifdef TIOCMODG
   {
     int mcs = 0;
@@ -212,10 +265,9 @@ static int m_word;
  */
 void m_savestate(int fd)
 {
-#ifdef USE_SOCKET
   if (portfd_is_socket)
     return;
-#endif
+
 #ifdef POSIX_TERMIOS
   tcgetattr(fd, &savetty);
 #else
@@ -237,12 +289,12 @@ void m_savestate(int fd)
  */
 void m_restorestate(int fd)
 {
-#ifdef USE_SOCKET
   if (portfd_is_socket)
     return;
-#endif
+
 #ifdef POSIX_TERMIOS
   tcsetattr(fd, TCSANOW, &savetty);
+  set_speed_apple(fd, &savetty);
 #else
 #  if defined(_BSD43) || defined(_V7)
   ioctl(fd, TIOCSETP, &sg);
@@ -264,10 +316,9 @@ void m_restorestate(int fd)
 /*ARGSUSED*/
 void m_nohang(int fd)
 {
-#ifdef USE_SOCKET
   if (portfd_is_socket)
     return;
-#endif
+
   {
 #ifdef POSIX_TERMIOS
     struct termios sgg;
@@ -275,6 +326,7 @@ void m_nohang(int fd)
     tcgetattr(fd, &sgg);
     sgg.c_cflag |= CLOCAL;
     tcsetattr(fd, TCSANOW, &sgg);
+    set_speed_apple(fd, &sgg);
 #else
 #  if defined (_BSD43) && defined(LNOHANG)
     int lsw;
@@ -292,10 +344,9 @@ void m_nohang(int fd)
  */
 void m_hupcl(int fd, int on)
 {
-#ifdef USE_SOCKET
   if (portfd_is_socket)
     return;
-#endif
+
   /* Eh, I don't know how to do this under BSD (yet..) */
 #ifdef POSIX_TERMIOS
   {
@@ -307,6 +358,7 @@ void m_hupcl(int fd, int on)
     else
       sgg.c_cflag &= ~HUPCL;
     tcsetattr(fd, TCSANOW, &sgg);
+    set_speed_apple(fd, &sgg);
   }
 #endif
 }
@@ -391,11 +443,11 @@ unsigned m_getmaxspd(void)
 /*
  * Set baudrate, parity and number of bits.
  */
-void m_setparms(int fd, char *baudr, char *par, char *bits, char *stopb,
-                int hwf, int swf)
+bool m_setparms(int fd, int newbaud, char *par, char *bits, char *stopb,
+                int hwf, int swf, int rs485en)
 {
   int spd = -1;
-  int newbaud;
+  unsigned c_cflag_cbaud = 0;
   int bit = bits[0];
 
 #ifdef POSIX_TERMIOS
@@ -404,10 +456,8 @@ void m_setparms(int fd, char *baudr, char *par, char *bits, char *stopb,
   struct sgttyb tty;
 #endif /* POSIX_TERMIOS */
 
-#ifdef USE_SOCKET
   if (portfd_is_socket)
-    return;
-#endif
+    return true;
 
 #ifdef POSIX_TERMIOS
   tcgetattr(fd, &tty);
@@ -416,14 +466,10 @@ void m_setparms(int fd, char *baudr, char *par, char *bits, char *stopb,
 #endif /* POSIX_TERMIOS */
 
 
-  /* We generate mark and space parity ourself. */
+  /* We generate mark and space parity ourselves. */
   if (bit == '7' && (par[0] == 'M' || par[0] == 'S'))
     bit = '8';
 
-  /* Check if 'baudr' is really a number */
-  if ((newbaud = (atol(baudr) / 100)) == 0 && baudr[0] != '0')
-    newbaud = -1;
-
   switch (newbaud) {
     case 0:
 #ifdef B0
@@ -432,75 +478,80 @@ void m_setparms(int fd, char *baudr, char *par, char *bits, char *stopb,
       spd = 0;
 #endif
       break;
-    case 3:	spd = B300;	break;
-    case 6:	spd = B600;	break;
-    case 12:	spd = B1200;	break;
-    case 24:	spd = B2400;	break;
-    case 48:	spd = B4800;	break;
-    case 96:	spd = B9600;	break;
+    case 300:	spd = B300;	break;
+    case 600:	spd = B600;	break;
+    case 1200:	spd = B1200;	break;
+    case 2400:	spd = B2400;	break;
+    case 4800:	spd = B4800;	break;
+    case 9600:	spd = B9600;	break;
 #ifdef B19200
-    case 192:	spd = B19200;	break;
+    case 19200:	spd = B19200;	break;
 #else /* B19200 */
 #  ifdef EXTA
-    case 192:	spd = EXTA;	break;
+    case 19200:	spd = EXTA;	break;
 #   else /* EXTA */
-    case 192:	spd = B9600;	break;
+    case 19200:	spd = B9600;	break;
 #   endif /* EXTA */
 #endif	 /* B19200 */
 #ifdef B38400
-    case 384:	spd = B38400;	break;
+    case 38400:	spd = B38400;	break;
 #else /* B38400 */
 #  ifdef EXTB
-    case 384:	spd = EXTB;	break;
+    case 38400:	spd = EXTB;	break;
 #   else /* EXTB */
-    case 384:	spd = B9600;	break;
+    case 38400:	spd = B9600;	break;
 #   endif /* EXTB */
 #endif	 /* B38400 */
 #ifdef B57600
-    case 576:	spd = B57600;	break;
+    case 57600:	spd = B57600;	break;
 #endif
 #ifdef B115200
-    case 1152:	spd = B115200;	break;
+    case 115200:	spd = B115200;	break;
 #endif
 #ifdef B230400
-    case 2304:	spd = B230400;	break;
+    case 230400:	spd = B230400;	break;
 #endif
 #ifdef B460800
-    case 4608: spd = B460800; break;
+    case 460800: spd = B460800; break;
 #endif
 #ifdef B500000
-    case 5000: spd = B500000; break;
+    case 500000: spd = B500000; break;
 #endif
 #ifdef B576000
-    case 5760: spd = B576000; break;
+    case 576000: spd = B576000; break;
 #endif
 #ifdef B921600
-    case 9216: spd = B921600; break;
+    case 921600: spd = B921600; break;
 #endif
 #ifdef B1000000
-    case 10000: spd = B1000000; break;
+    case 1000000: spd = B1000000; break;
 #endif
 #ifdef B1152000
-    case 11520: spd = B1152000; break;
+    case 1152000: spd = B1152000; break;
 #endif
 #ifdef B1500000
-    case 15000: spd = B1500000; break;
+    case 1500000: spd = B1500000; break;
 #endif
 #ifdef B2000000
-    case 20000: spd = B2000000; break;
+    case 2000000: spd = B2000000; break;
 #endif
 #ifdef B2500000
-    case 25000: spd = B2500000; break;
+    case 2500000: spd = B2500000; break;
 #endif
 #ifdef B3000000
-    case 30000: spd = B3000000; break;
+    case 3000000: spd = B3000000; break;
 #endif
 #ifdef B3500000
-    case 35000: spd = B3500000; break;
+    case 3500000: spd = B3500000; break;
 #endif
 #ifdef B4000000
-    case 40000: spd = B4000000; break;
+    case 4000000: spd = B4000000; break;
 #endif
+    default:
+		if (newbaud != -1) {
+		  spd = newbaud;
+		  c_cflag_cbaud = BOTHER;
+		}
   }
 
 #if defined (_BSD43) && !defined(POSIX_TERMIOS)
@@ -538,6 +589,8 @@ void m_setparms(int fd, char *baudr, char *par, char *bits, char *stopb,
 
 #ifdef POSIX_TERMIOS
 
+  tty.c_cflag = (tty.c_cflag & ~CBAUD) | c_cflag_cbaud;
+
   if (spd != -1) {
     cfsetospeed(&tty, (speed_t)spd);
     cfsetispeed(&tty, (speed_t)spd);
@@ -586,13 +639,86 @@ void m_setparms(int fd, char *baudr, char *par, char *bits, char *stopb,
     tty.c_cflag &= ~CSTOPB;
 
   tcsetattr(fd, TCSANOW, &tty);
+  set_speed_apple(fd, &tty);
 
-  m_setrts(fd);
+  // Check if the baud rate could be set
+  {
+    struct termios tty_check;
+    if (tcgetattr(fd, &tty_check))
+      return false;
+    if (cfgetospeed(&tty_check) != (speed_t)spd)
+      return false;
+    if (cfgetispeed(&tty_check) != (speed_t)spd)
+      return false;
+  }
+
+  if (!rs485en)
+    m_setrts(fd);
 #endif /* POSIX_TERMIOS */
 
 #ifndef _DCDFLOW
   m_sethwf(fd, hwf);
 #endif
+
+  return true;
+}
+
+void m_set485parms(int fd, int en, int rts_on_snd, int rts_aft_snd,
+                   int rx_dur_tx, int term_bus, char *del_rts_bef_snd,
+                   char *del_rts_aft_snd)
+{
+#if defined (SER_RS485_ENABLED) && defined (TIOCGRS485) && defined (TIOCSRS485)
+  struct serial_rs485 rs485conf;
+
+  if (ioctl(fd, TIOCGRS485, &rs485conf))
+    return;
+
+  if (en)
+    rs485conf.flags |= SER_RS485_ENABLED;
+  else
+    rs485conf.flags &= ~SER_RS485_ENABLED;
+
+  if (rts_on_snd)
+    rs485conf.flags |= SER_RS485_RTS_ON_SEND;
+  else
+    rs485conf.flags &= ~SER_RS485_RTS_ON_SEND;
+
+  if (rts_aft_snd)
+    rs485conf.flags |= SER_RS485_RTS_AFTER_SEND;
+  else
+    rs485conf.flags &= ~SER_RS485_RTS_AFTER_SEND;
+
+  rs485conf.delay_rts_before_send = atoi(del_rts_bef_snd);
+
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 36)
+  rs485conf.delay_rts_after_send = atoi(del_rts_aft_snd);
+#endif
+
+#ifdef SER_RS485_RX_DURING_TX
+  if (rx_dur_tx)
+    rs485conf.flags |= SER_RS485_RX_DURING_TX;
+  else
+    rs485conf.flags &= ~SER_RS485_RX_DURING_TX;
+#endif
+
+#ifdef SER_RS485_TERMINATE_BUS
+  if (term_bus)
+    rs485conf.flags |= SER_RS485_TERMINATE_BUS;
+  else
+    rs485conf.flags &= ~SER_RS485_TERMINATE_BUS;
+#endif
+
+  ioctl(fd, TIOCSRS485, &rs485conf);
+#else
+  (void)fd;
+  (void)en;
+  (void)rts_on_snd;
+  (void)rts_aft_snd;
+  (void)rx_dur_tx;
+  (void)term_bus;
+  (void)del_rts_bef_snd;
+  (void)del_rts_aft_snd;
+#endif
 }
 
 /*
