@@ -23,13 +23,12 @@
  * js&jl 04.98	the better filename selection window
  */
 
-#include <poll.h>
-
-#ifdef HAVE_CONFIG_H
 #include <config.h>
-#endif
-
+#include <poll.h>
 #include <wchar.h>
+#include <assert.h>
+#include <string.h>
+#include <sys/file.h>
 
 #include "port.h"
 #include "minicom.h"
@@ -135,7 +134,7 @@ static char *translate(char *s)
 
   translation = malloc(translation_length + 1);
   if (translation == NULL) {
-    do_log("out of memory");
+    do_log(_("out of memory"));
     return NULL;
   }
 
@@ -174,7 +173,7 @@ static char *translate(char *s)
 }
 
 /*
- * Trim the leading & trailing whitespaces from the string
+ * Trim leading & trailing whitespace from the string
  * jl 15.09.97
  */
 char *trim(char *outstring, char *instring, int n)
@@ -278,7 +277,7 @@ void updown(int what, int nr)
         return;
     }
     else if (P_PNN(g) == 'Y') {
-      s = input(_("Please enter file names"), buf);
+      s = input(_("Please enter file names"), buf, sizeof(buf));
       if (s == NULL)
         return;
     }
@@ -287,7 +286,7 @@ void updown(int what, int nr)
     cmdline_length = strlen(P_PPROG(g)) + strlen((char*) (multiple == 0 ? "" : s)) + 1; /* + 1 for ' ' */
     cmdline = malloc(cmdline_length + 1); /* + 1 for NUL */
     if (cmdline == NULL) {
-      werror(_("Out of memory: could allocate buffer for command line"));
+      werror(_("Out of memory: could not allocate buffer for command line"));
       return;
     }
     snprintf(cmdline, cmdline_length + 1, "%s %s", P_PPROG(g), multiple == 0 ? "" : s);
@@ -298,17 +297,19 @@ void updown(int what, int nr)
     do_log("%s", cmdline);   /* jl 22.06.97 */
 
   if (P_PFULL(g) == 'N') {
-    win = mc_wopen(10, 7, 70, 13, BSINGLE, stdattr, mfcolor, mbcolor, 1, 0, 1);
+    win = mc_wopen(5, 5, 74, 11, BSINGLE, stdattr, mfcolor, mbcolor, 1, 0, 1);
     snprintf(title, sizeof(title), _("%.30s %s - Press CTRL-C to quit"), P_PNAME(g),
              what == 'U' ? _("upload") : _("download"));
     mc_wtitle(win, TMID, title);
     if (pipe(pipefd) == -1)
-      werror("pipe() call failed");
+      werror(_("pipe() call failed"));
   } else
     mc_wleave();
 
   m_flush(portfd);
 
+  lockfile_remove();
+
   switch (udpid = fork()) {
     case -1:
       werror(_("Out of memory: could not fork()"));
@@ -334,8 +335,6 @@ void updown(int what, int nr)
           close(pipefd[1]);
       }
 
-      lockfile_remove();
-
       for (n = 1; n < _NSIG; n++)
         signal(n, SIG_DFL);
 
@@ -364,7 +363,7 @@ void updown(int what, int nr)
 #ifdef LOG_XFER
     xfl=fopen("xfer.log","wb");
 #endif
-    while ((n = read(pipefd[0], buf, sizeof(buf))) > 0) {
+    while ((n = read(pipefd[0], buf, sizeof(buf) - 1)) > 0) {
       buf[n] = '\0';
       mc_wputs(win, buf);
       timer_update();
@@ -385,7 +384,8 @@ void updown(int what, int nr)
             trim (trimbuf, buf, sizeof(trimbuf));
             do_log("%s", trimbuf);
           } else if (!strncmp (buffirst, "Bytes", 5)) {
-            strncpy (xfrstr, buf, sizeof(xfrstr));
+            _Static_assert(sizeof(xfrstr) >= sizeof(buf), "String sizes");
+            strcpy(xfrstr, buf);
           }
           buffirst[0] = 0;
           trimbuf[0] = 0;
@@ -413,7 +413,7 @@ void updown(int what, int nr)
   if (win == (WIN *)0)
     mc_wreturn();
 
-  lockfile_create(0);
+  lockfile_create();
 
   /* MARK updated 02/17/94 - Flush modem port before displaying READY msg */
   /* because a BBS often displays menu text right after a download, and we */
@@ -427,20 +427,8 @@ void updown(int what, int nr)
   timer_update();
 
   /* If we got interrupted, status != 0 */
-  if (win && (status & 0xFF00) == 0) {
-#if VC_MUSIC
-    if (P_SOUND[0] == 'Y') {
-      mc_wprintf(win, _("\n READY: press any key to continue..."));
-      music();
-    } else
-      sleep(1);
-#else
-    /* MARK updated 02/17/94 - If there was no VC_MUSIC capability, */
-    /* then at least make some beeps! */
-    if (P_SOUND[0] == 'Y')
-      mc_wprintf(win, "\007\007\007");
+  if (win && ((status & 0xFF00) == 0 || status == 1)) {
     sleep(1);
-#endif
   }
   if (win)
     mc_wclose(win, 1);
@@ -452,51 +440,70 @@ void lockfile_remove(void)
     return;
 
 #if !HAVE_LOCKDEV
-  if (lockfile[0])
+  if (lockfile_mode == Lockfile_mode_uucp)
     unlink(lockfile);
+  else
+    {
+      assert(lockfile_mode == Lockfile_mode_flock);
+      if (flock(portfd, LOCK_UN))
+        fprintf(stderr, _("Failed to free flock: %s\n"), strerror(errno));
+    }
 #else
   ttyunlock(dial_tty);
 #endif
 }
 
-int lockfile_create(int no_msgs)
+int lockfile_create(void)
 {
-  int n;
-
   if (portfd_is_socket)
     return 0;
 
+  assert(lockfile_mode != Lockfile_mode_unset);
+
 #if !HAVE_LOCKDEV
-  if (!lockfile[0])
-    return 0;
+  if (lockfile_mode == Lockfile_mode_uucp)
+    {
+      mode_t orig_umask = umask(022);
+      /* Create lockfile compatible with UUCP-1.2 */
+      int fd = open(lockfile, O_WRONLY | O_CREAT | O_EXCL, 0666);
+      umask(orig_umask);
+      if (fd >= 0) {
+        // FHS format:
+        char buf[12];
+        snprintf(buf, sizeof(buf),  "%10d\n", getpid());
+        buf[sizeof(buf) - 1] = 0;
+        if (write(fd, buf, strlen(buf)) < (ssize_t)strlen(buf))
+          fprintf(stderr, _("Failed to write lockfile %s\n"), lockfile);
+        close(fd);
+        return 0;
+      }
+
+      // Cannot create lockfile, use flocks
+      lockfile_mode = Lockfile_mode_flock;
+    }
+
+  // Check again, lockfile_mode might have changed to Lockfile_mode_flock
+  if (lockfile_mode == Lockfile_mode_flock)
+    {
+      int r = flock(portfd, LOCK_EX | LOCK_NB);
+      if (r < 0)
+        {
+          if (errno == -EWOULDBLOCK)
+            fprintf(stderr, _("%s is already locked.\n"), dial_tty);
+          else
+            fprintf(stderr, _("Failed to lock %s.\n"), dial_tty);
+          return -1;
+        }
+    }
 
-  int fd;
-  n = umask(022);
-  /* Create lockfile compatible with UUCP-1.2 */
-  if ((fd = open(lockfile, O_WRONLY | O_CREAT | O_EXCL, 0666)) < 0) {
-    if (!no_msgs)
-      werror(_("Cannot create lockfile!"));
-  } else {
-    // FHS format:
-    char buf[12];
-    snprintf(buf, sizeof(buf),  "%10d\n", getpid());
-    buf[sizeof(buf) - 1] = 0;
-    if (write(fd, buf, strlen(buf)) < (ssize_t)strlen(buf))
-      if (!no_msgs)
-        fprintf(stderr, _("Failed to write lockfile %s\n"), lockfile);
-    close(fd);
-  }
-  umask(n);
   return 0;
 #else
-  n = ttylock(dial_tty);
-  if (!no_msgs)
-    {
-      if (n < 0)
-        fprintf(stderr, _("Cannot create lockfile for %s: %s\n"), dial_tty, strerror(-n));
-      else if (n > 0)
-        fprintf(stderr, _("Device %s is locked.\n"), dial_tty);
-    }
+  int n = ttylock(dial_tty);
+  if (n < 0)
+    fprintf(stderr, _("Cannot create lockfile for %s: %s\n"),
+            dial_tty, strerror(-n));
+  else if (n > 0)
+    fprintf(stderr, _("Device %s is locked.\n"), dial_tty);
   return n;
 #endif
 }
@@ -513,7 +520,7 @@ void kermit(void)
   char *kermit_path = P_KERMIT;
 
   if (!kermit_path || !*kermit_path) {
-    werror("No kermit path defined!");
+    werror(_("No kermit path defined!"));
     return;
   }
 
@@ -525,11 +532,9 @@ void kermit(void)
       mc_wreturn();
       werror(_("Out of memory: could not fork()"));
       return;
-    case 0: /* Child */
-      close(portfd);
 
-      /* Remove lockfile */
-      lockfile_remove();
+    case 0: /* Child */
+      device_close();
 
       for (n = 0; n < _NSIG; n++)
         signal(n, SIG_DFL);
@@ -548,12 +553,6 @@ void kermit(void)
 
   /* Restore screen and keyboard modes */
   mc_wreturn();
-
-  /* Re-create lockfile */
-  lockfile_create(0);
-
-  m_flush(portfd);
-  port_init();
 }
 
 /* ============ Here begins the setenv function ============= */
@@ -633,8 +632,8 @@ void runscript(int ask, const char *s, const char *l, const char *p)
   int n, i;
   int pipefd[2];
   char buf[81];
-  char scr_lines[5];
-  char cmdline[128];
+  char scr_lines[7];
+  char cmdline[160];
   struct pollfd fds[2];
   char *translated_cmdline;
   char *ptr;
@@ -659,7 +658,7 @@ void runscript(int ask, const char *s, const char *l, const char *p)
     mc_wredraw(w, 1);
 
     while (!done) {
-      mc_wlocate(w, mbslen (question) + 5, 5);
+      mc_wlocate(w, mbswidth(question) + 5, 5);
       n = wxgetch();
       if (islower(n))
         n = toupper(n);
@@ -677,19 +676,19 @@ void runscript(int ask, const char *s, const char *l, const char *p)
           mc_wclose(w, 1);
           return;
         case 'A':
-          mc_wlocate(w, mbslen (username) + 1, 1);
+          mc_wlocate(w, mbswidth(username) + 1, 1);
           mc_wclreol(w);
           scr_user[0] = 0;
           mc_wgets(w, scr_user, 32, 32);
           break;
         case 'B':
-          mc_wlocate(w, mbslen (password) + 1, 2);
+          mc_wlocate(w, mbswidth(password) + 1, 2);
           mc_wclreol(w);
           scr_passwd[0] = 0;
           mc_wgets(w, scr_passwd, 32, 32);
           break;
         case 'C':
-          mc_wlocate(w, mbslen (name_of_script) + 1, 3);
+          mc_wlocate(w, mbswidth(name_of_script) + 1, 3);
           mc_wgets(w, scr_name, 32, sizeof(scr_name) - 1);
           break;
         default:
@@ -698,8 +697,11 @@ void runscript(int ask, const char *s, const char *l, const char *p)
     }
   } else {
     strncpy(scr_user, l, sizeof(scr_user));
+    scr_user[sizeof(scr_user) - 1] = '\0';
     strncpy(scr_name, s, sizeof(scr_name));
+    scr_name[sizeof(scr_name) - 1] = '\0';
     strncpy(scr_passwd, p, sizeof(scr_passwd));
+    scr_passwd[sizeof(scr_passwd) - 1] = '\0';
   }
   sprintf(scr_lines, "%d", (int) lines);  /* jl 13.09.97 */
 
@@ -772,7 +774,7 @@ void runscript(int ask, const char *s, const char *l, const char *p)
           if (i)
             vt_send(*ptr++);
           else
-            vt_out(*ptr++);
+            vt_out(*ptr++, 0);
         timer_update();
         mc_wflush();
       }
@@ -807,7 +809,6 @@ int paste_file(void)
   char buf[128] = "";
   char *ptr;
   int bytes_read;
-  unsigned long bdone = 0;
   int x;
 
   if ((s = filedir(1, 0)) == NULL)
@@ -819,7 +820,7 @@ int paste_file(void)
 
   while (fgets(line, sizeof(line), fp)) {
     /* Check for I/O or timer. */
-    x = check_io(portfd_connected, 0, 1000, buf, sizeof(buf), &bytes_read);
+    x = check_io_frontend(buf, sizeof(buf), &bytes_read);
 
     /*  Send data from the modem to the screen. */
     if ((x & 1)) {
@@ -827,7 +828,7 @@ int paste_file(void)
       while (bytes_read-- > 0) {
 	if (P_PARITY[0] == 'M' || P_PARITY[0] == 'S')
 	  *ptr &= 0x7f;
-	vt_out(*ptr++);
+	vt_out(*ptr++, 0);
       }
       mc_wflush();
     }
@@ -841,11 +842,9 @@ int paste_file(void)
 	vt_send(*s);
       vt_send('\r');
       vt_send('\n');
-      bdone += strlen(line) + 2;
     } else {
       for (s = line; *s; s++)
 	vt_send(*s);
-      bdone += strlen(s);
     }
     if (ldelay) {
 #ifdef HAVE_USLEEP
