@@ -23,11 +23,9 @@
  *    jl 06.07.98 use conversion tables with the capture file
  *    mark.einon@gmail.com 16/02/11 - Added option to timestamp terminal output
  */
-#ifdef HAVE_CONFIG_H
 #include <config.h>
-#endif
-
 #include <time.h>
+#include <stdint.h>
 #include "port.h"
 #include "minicom.h"
 #include "vt100.h"
@@ -43,6 +41,7 @@
  * 5 - ESC )
  * 6 - ESC #
  * 7 - ESC P
+ * 8 - ESC ]
  */
 static int esc_s = 0;
 
@@ -88,14 +87,13 @@ static struct escseq vt_keys[] = {
  * to default to no conversion.    jl 04.09.1997
  */
 
-unsigned char vt_inmap[256] = {
+unsigned char vt_inmap[256] __attribute__((nonstring)) = {
 #include "defmap.h"
 };
-unsigned char vt_outmap[256] = {
+unsigned char vt_outmap[256] __attribute__((nonstring)) = {
 #include "defmap.h"
 };
 
-#if TRANSLATE
 /* Taken from the Linux kernel source: linux/drivers/char/console.c */
 static char * vt_map[] = {
 /* 8-bit Latin-1 mapped to the PC character set: '.' means non-printable */
@@ -130,7 +128,6 @@ static char * vt_map[] = {
 };
 static char *vt_trans[2];
 static int vt_charset;          /* Character set. */
-#endif
 
 static int vt_echo;		/* Local echo on/off. */
 int vt_nl_delay;		/* Delay after CR key */
@@ -154,20 +151,18 @@ static int vt_docap;		/* Capture on/off. */
 static void (*vt_keyb)(int, int);/* Gets called for NORMAL/APPL switch. */
 static void (*termout)(const char *, int);/* Gets called to output a string. */
 
-static int escparms[8];		/* Cumulated escape sequence. */
+static int escparms[8];		/* Accumulated escape sequence. */
 static int ptr;                 /* Index into escparms array. */
-static long vt_tabs[5];		/* Tab stops for max. 32*5 = 160 columns. */
+static unsigned vt_tabs[5];	/* Tab stops for max. 32*5 = 160 columns. */
 
 static short newy1 = 0;		/* Current size of scrolling region. */
 static short newy2 = 23;
 
-/* Saved color and posistions */
+/* Saved color and positions */
 static short savex, savey, saveattr = XA_NORMAL, savecol = 112;
 
-#if TRANSLATE
 static short savecharset;
 static char *savetrans[2];
-#endif
 
 /*
  * Initialize the emulator once.
@@ -187,9 +182,9 @@ void vt_pinit(WIN *win, int fg, int bg)
   newy1 = 0;
   newy2 = vt_win->ys - 1;
   mc_wresetregion(vt_win);
-  if (fg > 0)
+  if (fg >= 0)
     vt_fg = fg;
-  if (bg > 0)
+  if (bg >= 0)
     vt_bg = bg;
   mc_wsetfgcol(vt_win, vt_fg);
   mc_wsetbgcol(vt_win, vt_bg);
@@ -225,11 +220,9 @@ void vt_init(int type, int fg, int bg, int wrap, int add_lf, int add_cr)
   vt_tabs[2] =
   vt_tabs[3] =
   vt_tabs[4] = 0x01010101;
-#if TRANSLATE
   vt_charset = 0;
   vt_trans[0] = savetrans[0] = vt_map[0];
   vt_trans[1] = savetrans[1] = vt_map[1];
-#endif
   ptr = 0;
   memset(escparms, 0, sizeof(escparms));
   esc_s = 0;
@@ -272,9 +265,9 @@ static void v_termout(const char *s, int len)
 
   if (vt_echo) {
     for (p = s; *p; p++) {
-      vt_out(*p);
+      vt_out(*p, 0);
       if (!vt_addlf && *p == '\r')
-        vt_out('\n');
+        vt_out('\n', 0);
     }
     mc_wflush();
   }
@@ -309,6 +302,9 @@ static void state1(int c)
     case 'P': /* ESC P (DCS, Device Control String) */
       esc_s = 7;
       return;
+    case ']': /* ESC ] (OSC, Operating System Command) */
+      esc_s = 8;
+      return;
     case 'D': /* Cursor down */
     case 'M': /* Cursor up */
       x = vt_win->curx;
@@ -336,19 +332,15 @@ static void state1(int c)
       savey = vt_win->cury;
       saveattr = vt_win->attr;
       savecol = vt_win->color;
-#if TRANSLATE
       savecharset = vt_charset;
       savetrans[0] = vt_trans[0];
       savetrans[1] = vt_trans[1];
-#endif
       break;
     case '8': /* Restore them */
     case 'u':
-#if TRANSLATE
       vt_charset = savecharset;
       vt_trans[0] = savetrans[0];
       vt_trans[1] = savetrans[1];
-#endif
       vt_win->color = savecol; /* HACK should use mc_wsetfgcol etc */
       mc_wsetattr(vt_win, saveattr);
       mc_wlocate(vt_win, savex, savey);
@@ -427,7 +419,7 @@ static void state2(int c)
   }
   /* Separation between numbers ? */
   if (c == ';') {
-    if (ptr < 15)
+    if (ptr < (int)ARRAY_SIZE(escparms) - 1)
       ptr++;
     return;
   }
@@ -539,18 +531,14 @@ static void state2(int c)
       savey = vt_win->cury;
       saveattr = vt_win->attr;
       savecol = vt_win->color;
-#if TRANSLATE
       savecharset = vt_charset;
       savetrans[0] = vt_trans[0];
       savetrans[1] = vt_trans[1];
-#endif
       break;
     case 'u': /* Restore them */
-#if TRANSLATE
       vt_charset = savecharset;
       vt_trans[0] = savetrans[0];
       vt_trans[1] = savetrans[1];
-#endif
       vt_win->color = savecol; /* HACK should use mc_wsetfgcol etc */
       mc_wsetattr(vt_win, saveattr);
       mc_wlocate(vt_win, savex, savey);
@@ -571,6 +559,12 @@ static void state2(int c)
         y += newy1;
       mc_wlocate(vt_win, x - 1, y - 1);
       break;
+    case 'G': /* HPA: Cursor to column x */
+    case '`':
+      if ((x = escparms[1]) == 0)
+        x = 1;
+      mc_wlocate(vt_win, x - 1, vt_win->cury);
+      break;
     case 'g': /* Clear tab stop(s) */
       if (escparms[0] == 0) {
         x = vt_win->curx;
@@ -678,6 +672,10 @@ static void state2(int c)
     case 'y': /* Self test modes */
     default:
       /* IGNORED */
+      if (0)
+        {
+          printf("minicom-debug: Unknown ESC sequence '%c'\n", c);
+        }
       break;
   }
   /* Ok, our escape sequence is all done */
@@ -712,6 +710,41 @@ static void dec_mode(int on_off)
       case 67: /* Backspace key sends. (FIXME: vt420) */
         /* setbackspace(on_off ? 8 : 127); */
         break;
+
+      case 47: /* Alternate screen */
+      case 1047:
+      case 1049:
+
+        if ((us_alternate && on_off) || !on_off)
+          {
+            vt_win = us;
+
+            mc_clear_window_simple(us_alternate);
+            mc_wclose(us_alternate, 1);
+            us_alternate = NULL;
+          }
+
+        if (on_off)
+          {
+            us_alternate = mc_wopen(0, 0, COLS - 1, us->y2, BNONE, XA_NORMAL,
+                                    tfcolor, tbcolor,  1, 0, 0);
+            vt_win = us_alternate;
+          }
+
+          {
+            char b[10];
+
+            snprintf(b, sizeof(b),
+                     "\e[?%d%c", escparms[i], on_off ? 'h' : 'l');
+            b[sizeof(b) - 1] = 0;
+
+            mc_wputs(stdwin, b);
+          }
+
+        if (on_off)
+          mc_clear_window_simple(us_alternate);
+
+        break;
       default: /* Mostly set up functions */
         /* IGNORED */
         break;
@@ -754,10 +787,6 @@ static void state3(int c)
 static void state4(int c)
 {
   /* Switch Character Sets. */
-#if !TRANSLATE
-  /* IGNORED */
-  (void)c;
-#else
   switch (c) {
     case 'A':
     case 'B':
@@ -768,7 +797,6 @@ static void state4(int c)
       vt_trans[0] = vt_map[1];
       break;
   }
-#endif
   esc_s = 0;
 }
 
@@ -778,10 +806,6 @@ static void state4(int c)
 static void state5(int c)
 {
   /* Switch Character Sets. */
-#if !TRANSLATE
-  /* IGNORED */
-  (void)c;
-#else
   switch (c) {
     case 'A':
     case 'B':
@@ -792,7 +816,6 @@ static void state5(int c)
       vt_trans[1] = vt_map[1];
       break;
   }
-#endif
   esc_s = 0;
 }
 
@@ -832,7 +855,7 @@ static void state6(int c)
 static void state7(int c)
 {
   /*
-   * Device dependant control strings. The Minix virtual console package
+   * Device dependent control strings. The Minix virtual console package
    * uses these sequences. We can only turn cursor on or off, because
    * that's the only one supported in termcap. The rest is ignored.
    */
@@ -871,6 +894,39 @@ static void state7(int c)
   buf[pos++] = c;
 }
 
+/*
+ * ESC ] Seen.
+ */
+static void state8(int c)
+{
+  /*
+   * Operating system commands.
+   * No support is currently implemented, they are simply thrown away.
+   * The sequences end with '\a' (BEL, terminal bell) or ESC-\ (ST).
+   */
+  static int state = 0;
+  switch (c) {
+    case 7:
+      /* Got BEL - done */
+      state = 0;
+      esc_s = 0;
+      return;
+    case ESC:
+      /* Possibly start of ST */
+      state = 1;
+      return;
+    case '\\':
+      /* Possibly end of ST */
+      if (state == 1) {
+	state = 0;
+	esc_s = 0;
+	return;
+      }
+      break;
+  };
+  state = 0;
+}
+
 static void output_s(const char *s)
 {
   mc_wputs(vt_win, s);
@@ -885,13 +941,12 @@ static void output_c(const char c)
     fputc(c, capfp);
 }
 
-void vt_out(int ch)
+void vt_out(int ch, wchar_t wc)
 {
   static unsigned char last_ch;
   int f;
   unsigned char c;
   int go_on = 0;
-  wchar_t wc;
 
   if (!ch)
     return;
@@ -900,17 +955,18 @@ void vt_out(int ch)
       && vt_line_timestamp != TIMESTAMP_LINE_OFF)
     {
       struct timeval tmstmp_now;
-      static time_t tmstmp_last;
+      static struct timeval tmstmp_last;
       char s[36];
       struct tm tmstmp_tm;
 
       gettimeofday(&tmstmp_now, NULL);
       if ((   vt_line_timestamp == TIMESTAMP_LINE_PER_SECOND
-           && tmstmp_now.tv_sec != tmstmp_last)
+           && tmstmp_now.tv_sec != tmstmp_last.tv_sec)
           || vt_line_timestamp == TIMESTAMP_LINE_SIMPLE
           || vt_line_timestamp == TIMESTAMP_LINE_EXTENDED)
         {
-          if (   localtime_r(&tmstmp_now.tv_sec, &tmstmp_tm)
+          if (   tmstmp_last.tv_sec
+              && localtime_r(&tmstmp_now.tv_sec, &tmstmp_tm)
               && strftime(s, sizeof(s), "[%F %T", &tmstmp_tm))
             {
               output_s(s);
@@ -920,7 +976,8 @@ void vt_out(int ch)
                   output_s("] ");
                   break;
                 case TIMESTAMP_LINE_EXTENDED:
-                  snprintf(s, sizeof(s), ".%03ld] ", tmstmp_now.tv_usec / 1000);
+                  snprintf(s, sizeof(s), ".%03ld] ",
+		           (unsigned long)tmstmp_now.tv_usec / 1000);
                   output_s(s);
                   break;
                 case TIMESTAMP_LINE_PER_SECOND:
@@ -928,7 +985,21 @@ void vt_out(int ch)
                   break;
                 };
             }
-          tmstmp_last = tmstmp_now.tv_sec;
+          tmstmp_last = tmstmp_now;
+        }
+      else if (vt_line_timestamp == TIMESTAMP_LINE_DELTA)
+        {
+          if (tmstmp_last.tv_sec)
+            {
+              unsigned long long d;
+              d =   (tmstmp_now.tv_sec * 1000000 + tmstmp_now.tv_usec)
+                  - (tmstmp_last.tv_sec * 1000000 + tmstmp_last.tv_usec);
+              snprintf(s, sizeof(s), "[%lld.%03lld] ",
+                       d / 1000000, (d % 1000000) / 1000);
+              s[sizeof(s) - 1] = 0;
+              output_s(s);
+            }
+          tmstmp_last = tmstmp_now;
         }
     }
 
@@ -955,7 +1026,7 @@ void vt_out(int ch)
     case '\t': /* Non - destructive TAB */
       /* Find next tab stop. */
       for (f = vt_win->curx + 1; f < 160; f++)
-        if (vt_tabs[f / 32] & (1 << f % 32))
+        if (vt_tabs[f / 32] & (1u << f % 32))
           break;
       if (f >= vt_win->xs)
         f = vt_win->xs - 1;
@@ -970,18 +1041,12 @@ void vt_out(int ch)
       mc_winclr(vt_win);
       mc_wlocate(vt_win, 0, 0);
       break;
-#if !TRANSLATE
-    case 14:
-    case 15:  /* Change character set. Not supported. */
-      break;
-#else
     case 14:
       vt_charset = 1;
       break;
     case 15:
       vt_charset = 0;
       break;
-#endif
     case 24:
     case 26:  /* Cancel escape sequence. */
       esc_s = 0;
@@ -996,10 +1061,15 @@ void vt_out(int ch)
       if(vt_addcr)
         mc_wputc(vt_win, '\r');
       output_c(c);
-	  break;
+      break;
     case '\b':
+      output_c(c); /* Backspace */
+      break;
     case 7: /* Bell */
-      output_c(c);
+      if (esc_s == 8)
+        go_on = 1;
+      else
+        output_c(c);
       break;
     default:
       go_on = 1;
@@ -1015,25 +1085,15 @@ void vt_out(int ch)
         fputc(P_CONVCAP[0] == 'Y' ? vt_inmap[c] : c, capfp);
       if (!using_iconv()) {
         c = vt_inmap[c];    /* conversion 04.09.97 / jl */
-#if TRANSLATE
         if (vt_type == VT100 && vt_trans[vt_charset] && vt_asis == 0)
           c = vt_trans[vt_charset][c];
-#endif
       }
-      /* FIXME: This is wrong, but making it right would require
-       * removing all the 8-bit mapping features. Assuming the locale
-       * is 8-bit, the character should not be changed by mapping to
-       * wchar and back; if the locale is multibyte, there is no hope
-       * of getting it right anyway. */
-      if (!using_iconv()) {
+      if (wc == 0)
         one_mbtowc (&wc, (char *)&c, 1); /* returns 1 */
-        if (vt_insert)
-          mc_winschar2(vt_win, wc, 1);
-        else
-          mc_wputc(vt_win, wc);
-      } else {
-        mc_wputc(vt_win, c);
-      }
+      if (vt_insert)
+        mc_winschar2(vt_win, wc, 1);
+      else
+        mc_wputc(vt_win, wc);
       break;
     case 1: /* ESC seen */
       state1(c);
@@ -1056,12 +1116,10 @@ void vt_out(int ch)
     case 7:
       state7(c);
       break;
+    case 8:
+      state8(c);
+      break;
   }
-
-  /* Flush output to capture file so that all output is visible there
-   * immediately. Causes a write syscall for every call though. */
-  if (capfp)
-    fflush(capfp);
 }
 
 /* Translate keycode to escape sequence. */
