@@ -18,13 +18,11 @@
  * 01.01.98 dickey@clark.net: fix for a history window closing bug
  * fmg 8/20/97: Added support for Search of History Buffer
  */
-#ifdef HAVE_CONFIG_H
 #include <config.h>
-#endif
-
 #include <limits.h>
 #include <stdarg.h>
 #include <wchar.h>
+#include <assert.h>
 
 #include "port.h"
 #include "minicom.h"
@@ -72,19 +70,19 @@ static const char *TS, *FS, *DS;
 #endif
 
 /* Special characters */
-static char D_UL;
-static char D_HOR;
-static char D_UR;
-static char D_LL;
-static char D_VER;
-static char D_LR;
+static unsigned char D_UL;
+static unsigned char D_HOR;
+static unsigned char D_UR;
+static unsigned char D_LL;
+static unsigned char D_VER;
+static unsigned char D_LR;
 
-static char S_UL;
-static char S_HOR;
-static char S_UR;
-static char S_LL;
-static char S_VER;
-static char S_LR;
+static unsigned char S_UL;
+static unsigned char S_HOR;
+static unsigned char S_UR;
+static unsigned char S_LL;
+static unsigned char S_VER;
+static unsigned char S_LR;
 
 static char _bufstart[BUFFERSIZE];
 static char *_bufpos = _bufstart;
@@ -336,9 +334,10 @@ static void _gotoxy(int x, int y)
  *                 0: only write to memory, not to screen
  *                 1: write to both screen and memory
  */
-static void _write(wchar_t c, int doit, int x, int y, char attr, char color)
+static int _write(wchar_t c, int doit, int x, int y, char attr, char color)
 {
   ELM *e;
+  int cwidth = 0;
 
   /* If the terminal has automatic margins, we can't write to the
    * last line, last character. After scrolling, this "invisible"
@@ -375,16 +374,20 @@ static void _write(wchar_t c, int doit, int x, int y, char attr, char color)
         for (i = 0; i < len; i++)
           outchar(buf[i]);
       }
+      cwidth = wcwidth(c);
 
-      curx++;
+      curx += cwidth;
     }
     if (doit >= 0) {
       e = &gmap[x + y * COLS];
       e->value = c;
       e->attr = attr;
       e->color = color;
+
+      cwidth = wcwidth(c);
     }
   }
+  return cwidth;
 }
 
 /*
@@ -592,6 +595,7 @@ void mc_wclose(WIN *win, int replace)
 {
   ELM *e;
   int x, y;
+  int cwidth;
 
   if (!win)
     return;
@@ -615,14 +619,16 @@ void mc_wclose(WIN *win, int replace)
       /* in multibyte characters such as Japanese -- from here */
       ELM *g;
       g = gmap + (y * stdwin->xs);
-      for (x = 0 ; x < win->x1; x++) {
-        _write(g->value, 1, x, y, g->attr, g->color);
-        g++;
+      for (x = 0 ; x < win->x1;) {
+        cwidth = _write(g->value, 1, x, y, g->attr, g->color);
+        x += cwidth;
+        g += cwidth;
       }
       /* to here */
-      for (x = win->x1; x <= win->x2; x++) {
-        _write(e->value, 1, x, y, e->attr, e->color);
-        e++;
+      for (x = win->x1; x <= win->x2;) {
+        cwidth = _write(e->value, 1, x, y, e->attr, e->color);
+        x += cwidth;
+        e += cwidth;
       }
     }
     _gotoxy(win->o_curx, win->o_cury);
@@ -705,6 +711,7 @@ void mc_wredraw(WIN *w, int newdirect)
   int minx, maxx, miny, maxy;
   ELM *e;
   int x, y;
+  int cwidth;
   int addcnt;
 
   minx = w->x1;
@@ -726,9 +733,10 @@ void mc_wredraw(WIN *w, int newdirect)
   e = gmap + (miny * stdwin->xs) + minx;
 
   for (y = miny; y <= maxy; y++) {
-    for(x = minx; x <= maxx; x++) {
-      _write(e->value, -1, x, y, e->attr, e->color);
-      e++;
+    for(x = minx; x <= maxx; ) {
+      cwidth = _write(e->value, -1, x, y, e->attr, e->color);
+      x += cwidth;
+      e += cwidth;
     }
     e += addcnt;
   }
@@ -1011,10 +1019,11 @@ void mc_wputc(WIN *win, wchar_t c)
       if (c != '\n') {
 	if (!win->wrap && win->curx >= win->xs)
 	  c = '>';
-        _write(c, win->direct, win->curx + win->x1,
-               win->cury + win->y1, win->attr, win->color);
-        if (++win->curx >= win->xs && !win->wrap) {
-          win->curx--;
+        int cwidth = _write(c, win->direct, win->curx + win->x1,
+                            win->cury + win->y1, win->attr, win->color);
+        win->curx += cwidth;
+        if (win->curx >= win->xs && !win->wrap) {
+          win->curx -= cwidth;
           curx = 0; /* Force to move */
           mv++;
         }
@@ -1046,15 +1055,16 @@ void mc_wdrawelm(WIN *w, int y, ELM *e)
  * 'accumulate' one line of ELM's into a string
  * WHY: need this in search function to see if line contains search pattern
  */
-void mc_wdrawelm_var(WIN *w, ELM *e, wchar_t *buf)
+void mc_wdrawelm_var(WIN *w, ELM *e, wchar_t **buf)
 {
-  int x, c = 0;
+  int sz = w->x2 - w->x1 + 2;
+  *buf = malloc(sizeof(**buf) * sz);
+  assert(*buf);
 
-  /* MARK updated 02/17/94 - Fixes bug, to do all 80 cols, not 79 cols */
-  for (x = w->x1; x <= w->x2; x++) {
-    buf[c++] = e->value;
-    e++;
-  }
+  (*buf)[sz - 1] = 0;
+
+  for (int c = 0; c < sz - 1; c++, e++)
+    (*buf)[c] = e->value;
 }
 
 /*
@@ -1492,6 +1502,10 @@ void mc_winclr(WIN *w)
 
 void mc_clear_window_simple(WIN *w)
 {
+    if (w == NULL) {
+        return;
+    }
+
   int x = 0, y = 0;
   _colson(us->color);
   _gotoxy(0, 0);
